"""
Autogenerated state module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

hub.exec.boto3.client.ec2.create_local_gateway_route_table_vpc_association
hub.exec.boto3.client.ec2.delete_local_gateway_route_table_vpc_association
hub.exec.boto3.client.ec2.describe_local_gateway_route_table_vpc_associations
"""



import copy
from typing import *
import dict_tools.differ as differ

async def present(hub, ctx, name: Text, local_gateway_route_table_id: Text, vpc_id: Text, tag_specifications: List = None)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Associates the specified VPC with the specified local gateway route table.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        local_gateway_route_table_id(Text): The ID of the local gateway route table.
        vpc_id(Text): The ID of the VPC.
        tag_specifications(List, optional): The tags to assign to the local gateway route table VPC association. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              aws_auto.ec2.local_gateway_route_table_vpc_association.present:
                - name: value
                - local_gateway_route_table_id: value
                - vpc_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.local_gateway_route_table_vpc_association.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    
    before = await hub.exec.boto3.client.ec2.describe_local_gateway_route_table_vpc_associations(name)

    if before:
        result["comment"] = f"'{name}' already exists"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.create_local_gateway_route_table_vpc_association(
                ctx,
                DryRun=ctx.test,
                
                **{"LocalGatewayRouteTableId": local_gateway_route_table_id, "VpcId": vpc_id, "TagSpecifications": tag_specifications}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            ret["comment"] = f"Created '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    
    # TODO perform other modifications as needed here
    ...

    after = await hub.exec.boto3.client.ec2.describe_local_gateway_route_table_vpc_associations(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def absent(hub, ctx, name: Text, local_gateway_route_table_vpc_association_id: Text)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Deletes the specified association between a VPC and local gateway route table.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        local_gateway_route_table_vpc_association_id(Text): The ID of the association.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              aws_auto.ec2.local_gateway_route_table_vpc_association.absent:
                - name: value
                - local_gateway_route_table_vpc_association_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.local_gateway_route_table_vpc_association.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    

    before = await hub.exec.boto3.client.ec2.describe_local_gateway_route_table_vpc_associations(name)

    if not before:
        result["comment"] = f"'{name}' already absent"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.delete_local_gateway_route_table_vpc_association(
                ctx,
                DryRun=ctx.test,
                
                **{"LocalGatewayRouteTableVpcAssociationId": local_gateway_route_table_vpc_association_id}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            result["comment"] = f"Deleted '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    

    after = await hub.exec.boto3.client.ec2.describe_local_gateway_route_table_vpc_associations(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def describe(hub, ctx)  -> Dict[str, Dict[str, Any]]:
    r'''
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function

    
    Describes the specified associations between VPCs and local gateway route tables.


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws_auto.ec2.local_gateway_route_table_vpc_association
    '''

    
    result = {}

    ret = await hub.exec.boto3.client.ec2.describe_local_gateway_route_table_vpc_associations(ctx)
    if not ret["status"]:
        hub.log.debug(f"Could not describe local_gateway_route_table_vpc_association {ret['comment']}")
        return result

    for local_gateway_route_table_vpc_association in ret["ret"]["TODO"]:
        new_local_gateway_route_table_vpc_association = [
                {"LocalGatewayRouteTableVpcAssociationIds": local_gateway_route_table_vpc_association_ids, "Filters": filters, "MaxResults": max_results, "NextToken": next_token}
        ]
        result[local_gateway_route_table_vpc_association["local_gateway_route_table_vpc_associationId"]] = {"aws_auto.ec2.local_gateway_route_table_vpc_association.present": new_local_gateway_route_table_vpc_association}

        for i, data in enumerate(local_gateway_route_table_vpc_association.get("", ())):
            sub_local_gateway_route_table_vpc_association = copy.deepcopy(new_local_gateway_route_table_vpc_association)

            # TODO check for subresouruces
            sub_local_gateway_route_table_vpc_association.append({})
            sub_local_gateway_route_table_vpc_association.append({"name": "TODOs"})
            result[f"TODOs-{i}"] = {"aws_auto.ec2.local_gateway_route_table_vpc_association.present": sub_local_gateway_route_table_vpc_association}

    return result


