"""
Autogenerated state module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

hub.exec.boto3.client.ec2.associate_transit_gateway_multicast_domain
hub.exec.boto3.client.ec2.create_transit_gateway_multicast_domain
hub.exec.boto3.client.ec2.delete_transit_gateway_multicast_domain
hub.exec.boto3.client.ec2.describe_transit_gateway_multicast_domains
hub.exec.boto3.client.ec2.disassociate_transit_gateway_multicast_domain
"""



import copy
from typing import *
import dict_tools.differ as differ

async def present(hub, ctx, name: Text, transit_gateway_id: Text, options: Dict = None, tag_specifications: List = None)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Creates a multicast domain using the specified transit gateway. The transit gateway must be in the available
    state before you create a domain. Use DescribeTransitGateways to see the state of transit gateway.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        transit_gateway_id(Text): The ID of the transit gateway.
        options(Dict, optional): The options for the transit gateway multicast domain. Defaults to None.
        tag_specifications(List, optional): The tags for the transit gateway multicast domain. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              aws_auto.ec2.transit_gateway_multicast_domain.present:
                - name: value
                - transit_gateway_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.transit_gateway_multicast_domain.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    
    before = await hub.exec.boto3.client.ec2.describe_transit_gateway_multicast_domains(name)

    if before:
        result["comment"] = f"'{name}' already exists"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.create_transit_gateway_multicast_domain(
                ctx,
                DryRun=ctx.test,
                
                **{"TransitGatewayId": transit_gateway_id, "Options": options, "TagSpecifications": tag_specifications}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            ret["comment"] = f"Created '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    
    # TODO perform other modifications as needed here
    ...

    after = await hub.exec.boto3.client.ec2.describe_transit_gateway_multicast_domains(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def absent(hub, ctx, name: Text, transit_gateway_multicast_domain_id: Text)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Deletes the specified transit gateway multicast domain.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        transit_gateway_multicast_domain_id(Text): The ID of the transit gateway multicast domain.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              aws_auto.ec2.transit_gateway_multicast_domain.absent:
                - name: value
                - transit_gateway_multicast_domain_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.transit_gateway_multicast_domain.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    

    before = await hub.exec.boto3.client.ec2.describe_transit_gateway_multicast_domains(name)

    if not before:
        result["comment"] = f"'{name}' already absent"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.delete_transit_gateway_multicast_domain(
                ctx,
                DryRun=ctx.test,
                
                **{"TransitGatewayMulticastDomainId": transit_gateway_multicast_domain_id}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            result["comment"] = f"Deleted '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    

    after = await hub.exec.boto3.client.ec2.describe_transit_gateway_multicast_domains(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def describe(hub, ctx)  -> Dict[str, Dict[str, Any]]:
    r'''
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function

    
    Describes one or more transit gateway multicast domains.


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws_auto.ec2.transit_gateway_multicast_domain
    '''

    
    result = {}

    ret = await hub.exec.boto3.client.ec2.describe_transit_gateway_multicast_domains(ctx)
    if not ret["status"]:
        hub.log.debug(f"Could not describe transit_gateway_multicast_domain {ret['comment']}")
        return result

    for transit_gateway_multicast_domain in ret["ret"]["TODO"]:
        new_transit_gateway_multicast_domain = [
                {"TransitGatewayMulticastDomainIds": transit_gateway_multicast_domain_ids, "Filters": filters, "MaxResults": max_results, "NextToken": next_token}
        ]
        result[transit_gateway_multicast_domain["transit_gateway_multicast_domainId"]] = {"aws_auto.ec2.transit_gateway_multicast_domain.present": new_transit_gateway_multicast_domain}

        for i, data in enumerate(transit_gateway_multicast_domain.get("", ())):
            sub_transit_gateway_multicast_domain = copy.deepcopy(new_transit_gateway_multicast_domain)

            # TODO check for subresouruces
            sub_transit_gateway_multicast_domain.append({})
            sub_transit_gateway_multicast_domain.append({"name": "TODOs"})
            result[f"TODOs-{i}"] = {"aws_auto.ec2.transit_gateway_multicast_domain.present": sub_transit_gateway_multicast_domain}

    return result


