"""
Autogenerated state module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

hub.exec.boto3.client.ec2.create_transit_gateway_prefix_list_reference
hub.exec.boto3.client.ec2.delete_transit_gateway_prefix_list_reference
hub.exec.boto3.client.ec2.get_transit_gateway_prefix_list_references
hub.exec.boto3.client.ec2.modify_transit_gateway_prefix_list_reference
"""



import copy
from typing import *
import dict_tools.differ as differ

async def present(hub, ctx, name: Text, transit_gateway_route_table_id: Text, prefix_list_id: Text, transit_gateway_attachment_id: Text = None, blackhole: bool = None)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Creates a reference (route) to a prefix list in a specified transit gateway route table.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        transit_gateway_route_table_id(Text): The ID of the transit gateway route table.
        prefix_list_id(Text): The ID of the prefix list that is used for destination matches.
        transit_gateway_attachment_id(Text, optional): The ID of the attachment to which traffic is routed. Defaults to None.
        blackhole(bool, optional): Indicates whether to drop traffic that matches this route. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              aws_auto.ec2.transit_gateway_prefix_list_reference.present:
                - name: value
                - transit_gateway_route_table_id: value
                - prefix_list_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.transit_gateway_prefix_list_reference.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    
    before = await hub.exec.boto3.client.ec2.get_transit_gateway_prefix_list_references(name)

    if before:
        result["comment"] = f"'{name}' already exists"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.create_transit_gateway_prefix_list_reference(
                ctx,
                DryRun=ctx.test,
                
                **{"TransitGatewayRouteTableId": transit_gateway_route_table_id, "PrefixListId": prefix_list_id, "TransitGatewayAttachmentId": transit_gateway_attachment_id, "Blackhole": blackhole}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            ret["comment"] = f"Created '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    
    # TODO perform other modifications as needed here
    ...

    after = await hub.exec.boto3.client.ec2.get_transit_gateway_prefix_list_references(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def absent(hub, ctx, name: Text, transit_gateway_route_table_id: Text, prefix_list_id: Text)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Deletes a reference (route) to a prefix list in a specified transit gateway route table.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        transit_gateway_route_table_id(Text): The ID of the route table.
        prefix_list_id(Text): The ID of the prefix list.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              aws_auto.ec2.transit_gateway_prefix_list_reference.absent:
                - name: value
                - transit_gateway_route_table_id: value
                - prefix_list_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.transit_gateway_prefix_list_reference.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    

    before = await hub.exec.boto3.client.ec2.get_transit_gateway_prefix_list_references(name)

    if not before:
        result["comment"] = f"'{name}' already absent"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.delete_transit_gateway_prefix_list_reference(
                ctx,
                DryRun=ctx.test,
                
                **{"TransitGatewayRouteTableId": transit_gateway_route_table_id, "PrefixListId": prefix_list_id}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            result["comment"] = f"Deleted '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    

    after = await hub.exec.boto3.client.ec2.get_transit_gateway_prefix_list_references(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def describe(hub, ctx)  -> Dict[str, Dict[str, Any]]:
    r'''
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function

    
    Gets information about the prefix list references in a specified transit gateway route table.


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws_auto.ec2.transit_gateway_prefix_list_reference
    '''

    
    result = {}

    ret = await hub.exec.boto3.client.ec2.get_transit_gateway_prefix_list_references(ctx)
    if not ret["status"]:
        hub.log.debug(f"Could not describe transit_gateway_prefix_list_reference {ret['comment']}")
        return result

    for transit_gateway_prefix_list_reference in ret["ret"]["TODO"]:
        new_transit_gateway_prefix_list_reference = [
                {"TransitGatewayRouteTableId": transit_gateway_route_table_id, "Filters": filters, "MaxResults": max_results, "NextToken": next_token}
        ]
        result[transit_gateway_prefix_list_reference["transit_gateway_prefix_list_referenceId"]] = {"aws_auto.ec2.transit_gateway_prefix_list_reference.present": new_transit_gateway_prefix_list_reference}

        for i, data in enumerate(transit_gateway_prefix_list_reference.get("", ())):
            sub_transit_gateway_prefix_list_reference = copy.deepcopy(new_transit_gateway_prefix_list_reference)

            # TODO check for subresouruces
            sub_transit_gateway_prefix_list_reference.append({})
            sub_transit_gateway_prefix_list_reference.append({"name": "TODOs"})
            result[f"TODOs-{i}"] = {"aws_auto.ec2.transit_gateway_prefix_list_reference.present": sub_transit_gateway_prefix_list_reference}

    return result


