"""
Autogenerated state module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

hub.exec.boto3.client.ec2.associate_transit_gateway_route_table
hub.exec.boto3.client.ec2.create_transit_gateway_route_table
hub.exec.boto3.client.ec2.delete_transit_gateway_route_table
hub.exec.boto3.client.ec2.describe_transit_gateway_route_tables
hub.exec.boto3.client.ec2.disassociate_transit_gateway_route_table
"""



import copy
from typing import *
import dict_tools.differ as differ

async def present(hub, ctx, name: Text, transit_gateway_id: Text, tag_specifications: List = None)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Creates a route table for the specified transit gateway.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        transit_gateway_id(Text): The ID of the transit gateway.
        tag_specifications(List, optional): The tags to apply to the transit gateway route table. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              aws_auto.ec2.transit_gateway_route_table.present:
                - name: value
                - transit_gateway_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.transit_gateway_route_table.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    
    before = await hub.exec.boto3.client.ec2.describe_transit_gateway_route_tables(name)

    if before:
        result["comment"] = f"'{name}' already exists"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.create_transit_gateway_route_table(
                ctx,
                DryRun=ctx.test,
                
                **{"TransitGatewayId": transit_gateway_id, "TagSpecifications": tag_specifications}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            ret["comment"] = f"Created '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    
    # TODO perform other modifications as needed here
    ...

    after = await hub.exec.boto3.client.ec2.describe_transit_gateway_route_tables(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def absent(hub, ctx, name: Text, transit_gateway_route_table_id: Text)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Deletes the specified transit gateway route table. You must disassociate the route table from any transit
    gateway route tables before you can delete it.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        transit_gateway_route_table_id(Text): The ID of the transit gateway route table.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              aws_auto.ec2.transit_gateway_route_table.absent:
                - name: value
                - transit_gateway_route_table_id: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.transit_gateway_route_table.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    

    before = await hub.exec.boto3.client.ec2.describe_transit_gateway_route_tables(name)

    if not before:
        result["comment"] = f"'{name}' already absent"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.delete_transit_gateway_route_table(
                ctx,
                DryRun=ctx.test,
                
                **{"TransitGatewayRouteTableId": transit_gateway_route_table_id}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            result["comment"] = f"Deleted '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    

    after = await hub.exec.boto3.client.ec2.describe_transit_gateway_route_tables(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def describe(hub, ctx)  -> Dict[str, Dict[str, Any]]:
    r'''
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function

    
    Describes one or more transit gateway route tables. By default, all transit gateway route tables are described.
    Alternatively, you can filter the results.


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws_auto.ec2.transit_gateway_route_table
    '''

    
    result = {}

    ret = await hub.exec.boto3.client.ec2.describe_transit_gateway_route_tables(ctx)
    if not ret["status"]:
        hub.log.debug(f"Could not describe transit_gateway_route_table {ret['comment']}")
        return result

    for transit_gateway_route_table in ret["ret"]["TODO"]:
        new_transit_gateway_route_table = [
                {"TransitGatewayRouteTableIds": transit_gateway_route_table_ids, "Filters": filters, "MaxResults": max_results, "NextToken": next_token}
        ]
        result[transit_gateway_route_table["transit_gateway_route_tableId"]] = {"aws_auto.ec2.transit_gateway_route_table.present": new_transit_gateway_route_table}

        for i, data in enumerate(transit_gateway_route_table.get("", ())):
            sub_transit_gateway_route_table = copy.deepcopy(new_transit_gateway_route_table)

            # TODO check for subresouruces
            sub_transit_gateway_route_table.append({})
            sub_transit_gateway_route_table.append({"name": "TODOs"})
            result[f"TODOs-{i}"] = {"aws_auto.ec2.transit_gateway_route_table.present": sub_transit_gateway_route_table}

    return result


