"""
Autogenerated state module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

hub.exec.boto3.client.ec2.accept_vpc_endpoint_connections
hub.exec.boto3.client.ec2.describe_vpc_endpoint_connections
hub.exec.boto3.client.ec2.reject_vpc_endpoint_connections
"""



import copy
from typing import *
import dict_tools.differ as differ

async def present(hub, ctx, name: Text, service_id: Text, vpc_endpoint_ids: List)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Accepts one or more interface VPC endpoint connection requests to your VPC endpoint service.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        service_id(Text): The ID of the VPC endpoint service.
        vpc_endpoint_ids(List): The IDs of one or more interface VPC endpoints.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              aws_auto.ec2.vpc_endpoint_connection.present:
                - name: value
                - service_id: value
                - vpc_endpoint_ids: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.vpc_endpoint_connection.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    
    before = await hub.exec.boto3.client.ec2.describe_vpc_endpoint_connections(name)

    if before:
        result["comment"] = f"'{name}' already exists"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.accept_vpc_endpoint_connections(
                ctx,
                DryRun=ctx.test,
                
                **{"ServiceId": service_id, "VpcEndpointIds": vpc_endpoint_ids}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            ret["comment"] = f"Created '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    
    # TODO perform other modifications as needed here
    ...

    after = await hub.exec.boto3.client.ec2.describe_vpc_endpoint_connections(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def absent(hub, ctx, name: Text, service_id: Text, vpc_endpoint_ids: List)  -> Dict[str, Any]:
    r'''
    **Autogenerated function**
    
    Rejects one or more VPC endpoint connection requests to your VPC endpoint service.

    Args:
        name(Text): A name, ID, or JMES search path to identify the resource.
        service_id(Text): The ID of the service.
        vpc_endpoint_ids(List): The IDs of one or more VPC endpoints.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              aws_auto.ec2.vpc_endpoint_connection.absent:
                - name: value
                - service_id: value
                - vpc_endpoint_ids: value
    '''

    
    result = dict(comment="", changes= None, name=name, result=True)
    ret = await hub.exec.boto3.client.ec2.vpc_endpoint_connection.id(
        ctx,
        jmes_path=name
    )
    if ret["status"]:
        # name is now the first id that matched the JMES search path
        name = ret["ret"]
    

    

    before = await hub.exec.boto3.client.ec2.describe_vpc_endpoint_connections(name)

    if not before:
        result["comment"] = f"'{name}' already absent"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.reject_vpc_endpoint_connections(
                ctx,
                DryRun=ctx.test,
                
                **{"ServiceId": service_id, "VpcEndpointIds": vpc_endpoint_ids}
            )
            result["result"] = ret["status"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            result["comment"] = f"Deleted '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    

    after = await hub.exec.boto3.client.ec2.describe_vpc_endpoint_connections(name)
    result["changes"] = differ.deep_diff(before, after)
    return result



async def describe(hub, ctx)  -> Dict[str, Dict[str, Any]]:
    r'''
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function

    
    Describes the VPC endpoint connections to your VPC endpoint services, including any endpoints that are pending
    your acceptance.


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws_auto.ec2.vpc_endpoint_connection
    '''

    
    result = {}

    ret = await hub.exec.boto3.client.ec2.describe_vpc_endpoint_connections(ctx)
    if not ret["status"]:
        hub.log.debug(f"Could not describe vpc_endpoint_connection {ret['comment']}")
        return result

    for vpc_endpoint_connection in ret["ret"]["TODO"]:
        new_vpc_endpoint_connection = [
                {"Filters": filters, "MaxResults": max_results, "NextToken": next_token}
        ]
        result[vpc_endpoint_connection["vpc_endpoint_connectionId"]] = {"aws_auto.ec2.vpc_endpoint_connection.present": new_vpc_endpoint_connection}

        for i, data in enumerate(vpc_endpoint_connection.get("", ())):
            sub_vpc_endpoint_connection = copy.deepcopy(new_vpc_endpoint_connection)

            # TODO check for subresouruces
            sub_vpc_endpoint_connection.append({})
            sub_vpc_endpoint_connection.append({"name": "TODOs"})
            result[f"TODOs-{i}"] = {"aws_auto.ec2.vpc_endpoint_connection.present": sub_vpc_endpoint_connection}

    return result


